﻿using IntelOrca.Launchpad;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using Timer = System.Timers.Timer;

namespace GGJ2015
{
    public class CounterAct : BaseGame
    {

        TileState[] grid;



        Random rand;

        List<Tile> tiles;

        
        List<Bullet> bullets;

        long currentTicks = 0;
        private bool isRunning;

        public override void Start(LaunchpadDevice[] players)
        {
            base.Start(players);
            rand = new Random();
            grid = Enumerable.Repeat(TileState.EMPTY, 64).ToArray();
            devices[0].Reset();
            devices[1].Reset();
            Task.Run(() =>
            {
                devices[0].RapidUpdate(grid);
                devices[1].RapidUpdate(grid);

            }).Wait();

            devices[0].DoubleBuffered = true;
            devices[1].DoubleBuffered = true;

            tiles = new List<Tile>();

            
            isRunning = true;

            makeBoard();

            long last_tick = Environment.TickCount;
            long delay = 12;

            

            while (isRunning)
            {
                if (Environment.TickCount - last_tick < delay)
                    continue;
                currentTicks += Environment.TickCount - last_tick;
                last_tick = Environment.TickCount;

                try
                {
                    update();
                    draw();
                }catch
                { 
                    //ah well, better luck NEXT frame
                }
            }

        }

        private void makeBoard()
        {
            List<Gun> guns = new List<Gun>();
            bullets = new List<Bullet>();

            for(int i = 0; i < 8; i++)
            {
                //player1
                var gun = new Gun()
                {
                    X = 0,
                    Y = i,
                    player = Player.ONE,
                    VX = 0,
                    VY = 0
                };
                guns.Add(gun);

                //player2
                gun = new Gun()
                {
                    X = 7,
                    Y = i,
                    player = Player.TWO,
                    VX = 0,
                    VY = 0
                };
                guns.Add(gun);

            }

            tiles.AddRange(guns.Concat<Tile>(bullets));

        }

        private void onEndGame(Player player)
        {
            isRunning = false;
            

            //Count scores

            //Display winners

            Task.Run(() =>
            {
                if(player == Player.TWO)
                {
                    devices[0].RapidUpdate(LaunchpadHelper.HappyFace);
                    devices[1].RapidUpdate(LaunchpadHelper.SadFace);
                }
                else if (player == Player.ONE)
                {
                    devices[0].RapidUpdate(LaunchpadHelper.SadFace);
                    devices[1].RapidUpdate(LaunchpadHelper.HappyFace);
                }
                

            }).Wait();

            Thread.Sleep(1000);
            Ended = true;
        }

        private void update()
        {
            tiles.ForEach(t =>
            {
                t.Update();
            });

            var bullets = tiles.Where(t => t.GetType().Equals(typeof(Bullet)));
            var guns = tiles.Where(t => t.GetType().Equals(typeof(Gun)));
            foreach(var group in bullets.GroupBy(b => String.Format("{0}{1}", (int)b.X, (int)b.Y)))
            {
                if (group.Count() > 1)
                {
                    group.ToList().ForEach(g =>
                        {
                            tiles.Remove(g);
                        });

                    //for (int i = 0; i < 3; i++)
                    //{
                    //    var t = new Tile()
                    //    {
                    //        X = group.First().X,
                    //        Y = group.First().Y,
                    //        VX = (rand.Next(2) == 0) ? 0.06 : 0.06 * -1,
                    //        VY = (rand.Next(2) == 0) ? 0.06 : 0.06 * -1,
                    //        Red = ButtonBrightness.Full, //(ButtonBrightness)rand.Next(4),
                    //        Green = ButtonBrightness.Full
                    //    };
                    //    tiles.Add(t);
                    //}
                }
            }

            var hitGuns = guns.Intersect(bullets, new GunEqualityComparer()).ToList();
            hitGuns.ForEach(g => { tiles.Remove(g); });

            //check for hitting shield;

            tiles.RemoveAll(t => t.LeftGrid);

            foreach(var gun in guns.GroupBy(g => g.player))
            {
                if (gun.Count() < 4)
                    onEndGame(gun.Key);
            }
        }

        private void draw()
        {
            ButtonBrightness[,] redgrid = new ButtonBrightness[8, 8];
            ButtonBrightness[,] greengrid = new ButtonBrightness[8, 8];
            
            tiles.ForEach(d =>
            {
                if (!(d.X >= 0 && d.X < 8 && d.Y >= 0 && d.Y < 8))
                    return;

                redgrid[(int)d.X, (int)d.Y] = d.Red;
                greengrid[(int)d.X, (int)d.Y] = d.Green;
            });

            for (int y = 0; y < 8; y++)
            {
                for (int x = 0; x < 8; x++)
                {
                    devices[0][x, y].SetBrightness(redgrid[x, y], greengrid[x, y]);
                    devices[1][x, y].SetBrightness(redgrid[x, y], greengrid[x, y]);
                }
            }
            devices[0].Refresh();
            devices[1].Refresh();
        }

        protected override void launchpadDevice0_ButtonPressed(object sender, ButtonPressEventArgs e)
        {
            var hit = tiles.Where(g => g.GetType() == typeof(Gun) && ((Gun)g).player.Equals(Player.ONE) && g.X == e.X && g.Y == e.Y);

            var velocity = 0.05;

            if(hit.Any())
            {
                //FIRE
                if(tiles.Where(g => g.GetType() == typeof(Bullet) && ((Bullet)g).player.Equals(Player.ONE)).Count() > 6 )
                    return;

                var bullet = new Bullet()
                {
                    X = e.X,
                    Y = e.Y,
                    VX = velocity,
                    VY = 0,
                    player = Player.ONE
                };

                tiles.Add(bullet);
            }

            
            //throw new NotImplementedException();
            
            //for(int i=0; i<5; i++)
            //{
            //    var t = new Tile()
            //    {
            //        X = e.X,
            //        Y = e.Y,
            //        VX = (rand.Next(2) == 0) ? (rand.NextDouble() / 10) : (rand.NextDouble() / 10)* -1,
            //        VY = (rand.Next(2) == 0) ? (rand.NextDouble() / 10) : (rand.NextDouble() / 10)* -1,
            //        Red = ButtonBrightness.Full, //(ButtonBrightness)rand.Next(4),
            //        Green = (ButtonBrightness)rand.Next(2)
            //    };
            //    tiles.Add(t);
            //}
            
        }

        protected override void launchpadDevice1_ButtonPressed(object sender, ButtonPressEventArgs e)
        {
            var hit = tiles.Where(g => g.GetType() == typeof(Gun) && ((Gun)g).player.Equals(Player.TWO) && g.X == e.X && g.Y == e.Y);

            var velocity = 0.05;

            if (hit.Any())
            {
                //FIRE
                if (tiles.Where(g => g.GetType() == typeof(Bullet) && ((Bullet)g).player.Equals(Player.TWO)).Count() > 6)
                    return;

                var bullet = new Bullet()
                {
                    X = e.X,
                    Y = e.Y,
                    VX = -velocity,
                    VY = 0,
                    player = Player.TWO
                };

                tiles.Add(bullet);
            }
        }
    }
}
