﻿using IntelOrca.Launchpad;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using Timer = System.Timers.Timer;

namespace GGJ2015.Games
{
    public class DBTest : BaseGame
    {
        Timer timer;


        TileState[] grid;

        Random rand;

        List<Tile> tiles;
        long currentTicks = 0;
        private bool isRunning;

        public override void Start(LaunchpadDevice[] players)
        {
            base.Start(players);
            rand = new Random();
            grid = Enumerable.Repeat(TileState.EMPTY, 64).ToArray();
            devices[0].Reset();
            devices[1].Reset();
            Task.Run(() =>
            {
                devices[0].RapidUpdate(grid);
                devices[1].RapidUpdate(grid);

            }).Wait();

            devices[0].DoubleBuffered = true;
            devices[1].DoubleBuffered = true;

            tiles = new List<Tile>();

            //Set up Timer
            timer = new Timer(10000);
            timer.Elapsed += onEndGame;
            timer.Enabled = true;
            isRunning = true;

            long last_tick = Environment.TickCount;
            long delay = 12;

            while (isRunning)
            {
                if (Environment.TickCount - last_tick < delay)
                    continue;
                currentTicks += Environment.TickCount - last_tick;
                last_tick = Environment.TickCount;

                try
                {
                    update();
                    draw();
                }catch
                { 
                    //ah well, better luck NEXT frame
                }
            }

        }


        private void onEndGame(object sender, ElapsedEventArgs e)
        {
            isRunning = false;
            timer.Enabled = false;

            //Count scores

            //Display winners

            Task.Run(() =>
            {

                devices[0].RapidUpdate(LaunchpadHelper.HappyFace);
                devices[1].RapidUpdate(LaunchpadHelper.HappyFace);

            }).Wait();

            Thread.Sleep(1000);
            Ended = true;
        }

        private void update()
        {
            tiles.ForEach(t =>
                {
                    t.Update();
                });

            tiles.RemoveAll(t => t.LeftGrid);

            draw();

        }

        private void draw()
        {
            ButtonBrightness[,] redgrid = new ButtonBrightness[8, 8];
            ButtonBrightness[,] greengrid = new ButtonBrightness[8, 8];
            
            tiles.ForEach(d =>
            {
                if (!(d.X >= 0 && d.X < 8 && d.Y >= 0 && d.Y < 8))
                    return;

                redgrid[(int)d.X, (int)d.Y] = d.Red;
                greengrid[(int)d.X, (int)d.Y] = d.Green;
            });

            for (int y = 0; y < 8; y++)
            {
                for (int x = 0; x < 8; x++)
                {
                    devices[0][x, y].SetBrightness(redgrid[x, y], greengrid[x, y]);
                    devices[1][x, y].SetBrightness(redgrid[x, y], greengrid[x, y]);
                }
            }
            devices[0].Refresh();
            devices[1].Refresh();
        }

        protected override void launchpadDevice0_ButtonPressed(object sender, ButtonPressEventArgs e)
        {
            //throw new NotImplementedException();
            
            for(int i=0; i<5; i++)
            {
                var t = new Tile()
                {
                    X = e.X,
                    Y = e.Y,
                    VX = (rand.Next(2) == 0) ? (rand.NextDouble() / 10) : (rand.NextDouble() / 10)* -1,
                    VY = (rand.Next(2) == 0) ? (rand.NextDouble() / 10) : (rand.NextDouble() / 10)* -1,
                    Red = ButtonBrightness.Full, //(ButtonBrightness)rand.Next(4),
                    Green = (ButtonBrightness)rand.Next(2)
                };
                tiles.Add(t);
            }
            
        }

        protected override void launchpadDevice1_ButtonPressed(object sender, ButtonPressEventArgs e)
        {
            for (int i = 0; i < 5; i++)
            {
                var t = new Tile()
                {
                    X = e.X,
                    Y = e.Y,
                    VX = (rand.Next(2) == 0) ? (rand.NextDouble() / 10) : (rand.NextDouble() / 10) * -1,
                    VY = (rand.Next(2) == 0) ? (rand.NextDouble() / 10) : (rand.NextDouble() / 10) * -1,
                    Red = (ButtonBrightness)rand.Next(2),
                    Green = ButtonBrightness.Full, //(ButtonBrightness)rand.Next(4)
                };
                tiles.Add(t);
            }
        }
    }
}
