﻿using IntelOrca.Launchpad;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using Timer = System.Timers.Timer;

namespace GGJ2015.Games
{
    public class GameOfLife : BaseGame
    {
        Timer timer;
        
        // Holds the current state of the board.
        private bool[,] board;
        private int h = 8;
        private int w = 8;

        private int delay = 200;

        bool isRunning;

        private Random rand;
        
        public override void Start(LaunchpadDevice[] players)
        {
            devices = players;
            Ended = false;
            

            rand = new Random();

            devices[0].DoubleBuffered = true;
            devices[0].ButtonPressed += launchpadDevice0_ButtonPressed;

            devices[1].DoubleBuffered = true;
            devices[1].ButtonPressed += launchpadDevice1_ButtonPressed;

            initBoard();
            isRunning = true;

            devices[0].Reset();
            devices[1].Reset();

            //Set up Timer
            timer = new Timer(20000);
            timer.Elapsed += onEndGame;
            timer.Enabled = true;
            isRunning = true;

        }

        private void onEndGame(object sender, ElapsedEventArgs e)
        {
            isRunning = false;
            timer.Enabled = false;

            Task.Run(() =>
            {
                    devices[0].RapidUpdate(LaunchpadHelper.HappyFace);
                    devices[1].RapidUpdate(LaunchpadHelper.HappyFace);
                
            }).Wait();

            Thread.Sleep(1000);
            Ended = true;
        }


        public override void Update()
        {
            if (!isRunning)
                return;
            //draw board;
            for (int y = 0; y < 8; y++)
            {
                for (int x = 0; x < 8; x++)
                {
                    devices[0][x, y].SetBrightness(
                        board[x, y] ? ButtonBrightness.Full : ButtonBrightness.Off,
                        board[x, y] ? ButtonBrightness.Full : ButtonBrightness.Off);

                    devices[1][x, y].SetBrightness(
                        board[x, y] ? ButtonBrightness.Full : ButtonBrightness.Off,
                        board[x, y] ? ButtonBrightness.Full : ButtonBrightness.Off);
                }
            }

            updateBoard();

            Thread.Sleep(delay);

        }


        protected override void launchpadDevice0_ButtonPressed(object sender, IntelOrca.Launchpad.ButtonPressEventArgs e)
        {
            if(e.Type.Equals(ButtonType.Grid))
                board[e.X, e.Y] = true;
        }

        protected override void launchpadDevice1_ButtonPressed(object sender, IntelOrca.Launchpad.ButtonPressEventArgs e)
        {
            if (e.Type.Equals(ButtonType.Grid))
                board[e.X, e.Y] = true;
        }

        #region GOL

        private void initBoard()
        {
            board = new bool[h, w];
            for (var y = 0; y < h; y++)
            {
                for (var x = 0; x < h; x++)
                {
                    // Equal probability of being true or false.
                    board[x, y] = rand.Next(2) == 0;
                }
            }
        }

        // Moves the board to the next state based on Conway's rules.
        private void updateBoard()
        {
            // A temp variable to hold the next state while it's being calculated.
            bool[,] newBoard = new bool[w, h];

            for (var y = 0; y < h; y++)
            {
                for (var x = 0; x < w; x++)
                {
                    var n = countLiveNeighbors(x, y);
                    var c = board[x, y];

                    // A live cell dies unless it has exactly 2 or 3 live neighbors.
                    // A dead cell remains dead unless it has exactly 3 live neighbors.
                    newBoard[x, y] = c && (n == 2 || n == 3) || !c && n == 3;
                }
            }

            // Set the board to its new state.
            board = newBoard;
        }

        // Returns the number of live neighbors around the cell at position (x,y).
        private int countLiveNeighbors(int x, int y)
        {
            // The number of live neighbors.
            int value = 0;

            // This nested loop enumerates the 9 cells in the specified cells neighborhood.
            for (var j = -1; j <= 1; j++)
            {
                // Loop around the edges if j is off the board.
                int k = (y + j + h) % h;

                for (var i = -1; i <= 1; i++)
                {
                    // Loop around the edges if i is off the board.
                    int l = (x + i + w) % w;

                    // Count the neighbor cell at (h,k) if it is alive.
                    value += board[l, k] ? 1 : 0;
                }
            }

            // Subtract 1 if (x,y) is alive since we counted it as a neighbor.
            return value - (board[x, y] ? 1 : 0);
        }

        #endregion
    }
}
