﻿using IntelOrca.Launchpad;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using Timer = System.Timers.Timer;

namespace GGJ2015
{
    public class TwoPresses : BaseGame
    {
        Timer timer;
        bool isRunning;
        TileState[] grid;

        public override void Start(LaunchpadDevice[] players)
        {
            base.Start(players);

            grid = Enumerable.Repeat(TileState.EMPTY, 64).ToArray();
            devices[0].Reset();
            devices[1].Reset();

            //Set up Timer
            timer = new Timer(10000);
            timer.Elapsed += onEndGame;
            timer.Enabled = true;
            isRunning = true;

        }

        private void onEndGame(object sender, ElapsedEventArgs e)
        {
            isRunning = false;
            timer.Enabled = false;

            //Count scores

            //Display winners

            Task.Run(() =>
            {
                if (grid.All(g => g.Equals(TileState.BOTH)))
                {
                    devices[0].RapidUpdate(LaunchpadHelper.HappyFace);
                    devices[1].RapidUpdate(LaunchpadHelper.HappyFace);
                }
                else
                {
                    devices[1].RapidUpdate(LaunchpadHelper.SadFace);
                    devices[0].RapidUpdate(LaunchpadHelper.SadFace);
                }
            }).Wait();

            Thread.Sleep(1000);
            Ended = true;
        }


        protected override void launchpadDevice0_ButtonPressed(object sender, IntelOrca.Launchpad.ButtonPressEventArgs e)
        {
            update(e, TileState.PLAYER1);
        }

        protected override void launchpadDevice1_ButtonPressed(object sender, IntelOrca.Launchpad.ButtonPressEventArgs e)
        {
            update(e, TileState.PLAYER2);
        }

        private void update(ButtonPressEventArgs e, TileState player)
        {
            if (!isRunning)
                return;

            if (e.Type == ButtonType.Grid)
            {
                //set grid array
                var curGrid = grid[e.ToPos()];

                if (curGrid == TileState.BOTH)
                    return;
                else if (curGrid == TileState.EMPTY)
                    grid[e.ToPos()] = player;
                else if((curGrid.Equals(TileState.PLAYER1) && !player.Equals(TileState.PLAYER1)) ||
                    (curGrid.Equals(TileState.PLAYER2) && !player.Equals(TileState.PLAYER2)))
                    grid[e.ToPos()] = TileState.BOTH;

                //update light value for that item.
                setFromGrid(e, 1);
                setFromGrid(e);
            }
        }

        protected void setFromGrid(ButtonPressEventArgs e, int deviceIdx = 0)
        {
            //update light value for that item.
            LaunchpadButton button = devices[deviceIdx][e.X, e.Y];

            switch (grid[e.ToPos()])
            {
                case TileState.BOTH:
                    button.TurnOnLight();
                    break;
                case TileState.PLAYER1:
                    button.SetBrightness(ButtonBrightness.Full, ButtonBrightness.Off);
                    break;
                case TileState.PLAYER2:
                    button.SetBrightness(ButtonBrightness.Off, ButtonBrightness.Full);
                    break;
                default:
                    button.TurnOffLight();
                    break;
            }
        }


    }
}
