﻿using IntelOrca.Launchpad;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using Timer = System.Timers.Timer;

namespace GGJ2015
{
    public class TwoPressesExtreme : BaseGame
    {
        Timer timer;
        bool isRunning;
        TileState[] grid;

        public override void Start(LaunchpadDevice[] players)
        {
            base.Start(players);

            grid = Enumerable.Repeat(TileState.EMPTY, 64).ToArray();
            devices[0].Reset();
            devices[1].Reset();

            //Set up Timer
            timer = new Timer(10000);
            timer.Elapsed += onEndGame;
            timer.Enabled = true;
            isRunning = true;

        }

        private void onEndGame(object sender, ElapsedEventArgs e)
        {
            isRunning = false;
            timer.Enabled = false;

            //Count scores

            //Display winners
            var p1 = grid.Count(t => t.Equals(TileState.PLAYER1));
            var p2 = grid.Count(t => t.Equals(TileState.PLAYER2));
            Task.Run(() =>
            {
                
                if (p1 > p2)
                {
                    devices[0].RapidUpdate(LaunchpadHelper.HappyFace);
                    devices[1].RapidUpdate(LaunchpadHelper.SadFace);
                }
                else if (p2 > p1)
                {
                    devices[1].RapidUpdate(LaunchpadHelper.HappyFace);
                    devices[0].RapidUpdate(LaunchpadHelper.SadFace);
                }
            }).Wait();

            Thread.Sleep(1000);
            Ended = true;
        }


        protected override void launchpadDevice0_ButtonPressed(object sender, IntelOrca.Launchpad.ButtonPressEventArgs e)
        {
            update(e, TileState.PLAYER1);
        }

        protected override void launchpadDevice1_ButtonPressed(object sender, IntelOrca.Launchpad.ButtonPressEventArgs e)
        {
            update(e, TileState.PLAYER2);
        }

        private void update(ButtonPressEventArgs e, TileState player)
        {
            if (!isRunning)
                return;

            if (e.Type == ButtonType.Grid)
            {
                //set grid array
                var curGrid = grid[e.ToPos()];

                switch (curGrid)
                {
                    case TileState.PLAYER1:
                        if (player.Equals(TileState.PLAYER2))
                            grid[e.ToPos()] = TileState.BOTH;
                        break;
                    case TileState.PLAYER2:
                        if (player.Equals(TileState.PLAYER1))
                            grid[e.ToPos()] = TileState.BOTH;
                        break;
                    case TileState.EMPTY:
                        grid[e.ToPos()] = player;
                        break;
                    case TileState.BOTH:
                        grid[e.ToPos()] = TileState.EMPTY;
                        break;
                }

                //update light value for that item.
                setFromGrid(e, 1);
                setFromGrid(e);
            }
        }

        protected void setFromGrid(ButtonPressEventArgs e, int deviceIdx = 0)
        {
            //update light value for that item.
            LaunchpadButton button = devices[deviceIdx][e.X, e.Y];

            switch (grid[e.ToPos()])
            {
                case TileState.BOTH:
                    button.TurnOnLight();
                    break;
                case TileState.PLAYER1:
                    button.SetBrightness(ButtonBrightness.Full, ButtonBrightness.Off);
                    break;
                case TileState.PLAYER2:
                    button.SetBrightness(ButtonBrightness.Off, ButtonBrightness.Full);
                    break;
                default:
                    button.TurnOffLight();
                    break;
            }
        }
    }
}
